/*
   Copyright (c) 2004-2005 The University of Chicago, as Operator
   of Argonne National Laboratory.
*/
package CFWDemo;

import java.net.URL;
import java.util.List;
import java.util.Set;
import java.util.Iterator;

// XAL/CFW framework stuff
import gov.sns.application.*;

// Work queue support from concurrent.jar
import EDU.oswego.cs.dl.util.concurrent.QueuedExecutor;

// Utility class for defining units of background thread work
import gov.anl.aps.irmis.apps.SwingThreadWork;

// irmisBase db imports. Just getting ioc list for this demo.
import gov.anl.aps.irmis.persistence.pv.IOCDAO;   // IOC Data Access Object
import gov.anl.aps.irmis.persistence.DAOException;


/**
 * The primary document and controller class for the Demo application. 
 * Provides "action" methods to query the database for data, or whatever you wish to do.
 * These action methods are typically invoked by the GUI in <code>DemoWindow</code>.
 * Any time-consuming activity should be queued up using the 
 * <code>queuedExecutor</code>, as shown below.
 */
public class DemoDocument extends XalInternalDocument {

	/** The main data model of this application */
	private DemoModel _model;

    /** 
     * Background work queue we use to handle lengthy logic (ie. sql queries). This
     * is a work queue backed by a single common background thread. It's important
     * to conduct most if not all sql activity on this thread, as the Hibernate
     * session is maintained in ThreadLocal storage, and therefore should only
     * exist on one thread to take advantage of Hibernate caching. The QueuedExecutor
     * class comes from the public domain concurrent.jar library, most of which
     * went into Sun's JDK 1.5 java.util.concurrent package. We're still on JDK1.4.
     */
	private QueuedExecutor queuedExecutor = new QueuedExecutor();

	/*********************** Document Constructors ******************************/

	/** 
     * Create a new empty document. Invoked by menubar "New", or when Desktop first
     * starts up.
     */
    public DemoDocument() {
        this(null);
    }

    /**
     * Create document from url. This would get invoked by CFW if you "opened"
     * a file that this document had previously saved.
     */
    public DemoDocument(final URL url) {
        _model = new DemoModel();

        // don't have anything here yet
    }

    /*********************** CFW required methods  ******************************/    

    /**
     * This handles a menubar "Save" or "Save As...". We don't do anything with
     * document save/open yet.
     */
    public void saveDocumentAs(final URL url) {
        return;
    }
    
            
    /**
     * Make the GUI that is the View of our Model.
     */
    public void makeMainWindow() {

        _mainWindow = new DemoWindow(this);

    }

	
	/**
	 * Get the main model of this document
	 * @return the main model of this document
	 */
	public DemoModel getModel() {
		return _model;
	}

    /************************** ACTION (Controller) METHODS *********************/
    /**** Add your application specific activities here *************/

    /**
     * Action method which is typically invoked by the GUI (DemoWindow) class.
     * Queue up job for background thread to gather the current set of IOC objects.
     * When complete, fire off <code>DemoModelEvent</code> to notify GUI (DemoWindow)
     * that new list of iocs is available for display. During this
     * job, the main window cursor will change to an hourglass icon.
     */
    public void actionIOCSearch() {
        try {
            /**
             * Queue up a chunk of work defined as a SwingThreadWork instance.
             * Note the _mainWindow argument just tells where the "busy" hourglass
             * cursor should show up while this stuff is executed.
             */
            queuedExecutor.execute(new SwingThreadWork(_mainWindow) {

                    /**
                     * Do time-consuming work in this method. In this case, we
                     * conduct the database query here. This method will be run
                     * in a background thread. Put the results of your work into
                     * the _model at the end of the method.
                     */
                    public void doNonUILogic() {
                        // Get current set of IOC objects, and put into DemoModel
                        IOCDAO iocDAO = null;
                        List iocList = null;
                        try {
                            iocDAO = new IOCDAO();
                            iocList = iocDAO.findIOCs();
                        } catch (DAOException de) {
                            de.printStackTrace();
                            // log error
                        }
                        _model.setIocList(iocList);
                    }

                    /**
                     * Use this method to tell model it needs to notify listeners
                     * that it has new data. This method actually winds up getting
                     * executed on the Swing event queue thread.
                     */
                    public void doUIUpdateLogic() {
                        // fire off notification of modification to listeners
                        _model.notifyDemoModelListeners(new DemoModelEvent(DemoModelEvent.NEW_IOC_LIST));
                    }
                });
        } catch (InterruptedException ie) {}
    }


    /************************** PRIVATE METHODS *******************************/

    /**
	 * Convenience method for getting the main window cast to the proper subclass of XalWindow.
     * This allows me to avoid casting the window every time I reference it.
     * @return The main window cast to its dynamic runtime class
     */
    private DemoWindow myWindow() {
        return (DemoWindow)_mainWindow;
    }
}
