/*
   Copyright (c) 2004-2005 The University of Chicago, as Operator
   of Argonne National Laboratory.
*/
package gov.anl.aps.irmis.apps.ioc.cfw;

import java.net.URL;
import java.util.List;
import java.util.Set;
import java.util.Iterator;
import javax.swing.SwingUtilities;
import javax.swing.JDesktopPane;
import javax.swing.JFrame;

// XAL framework stuff
import gov.sns.application.*;

// general app utilities
import gov.anl.aps.irmis.apps.SwingThreadWork;
import gov.anl.aps.irmis.apps.QueuedExecutorUtil;

// rdbCore imports
import gov.anl.aps.irmis.persistence.pv.IOCDAO;
import gov.anl.aps.irmis.persistence.pv.IOCAttributeTypeDAO;
import gov.anl.aps.irmis.persistence.pv.IOCAttributeDAO;
import gov.anl.aps.irmis.persistence.pv.IOC;
import gov.anl.aps.irmis.persistence.pv.IOCAttribute;
import gov.anl.aps.irmis.persistence.DAOException;


/**
 * The primary controller class for IRMIS IOC application.
 * Provides "action" methods to to query the IRMIS database for ioc's and
 * related data. These action methods are typically invoked by the 
 * <code>IOCWindow</code>.
 * Any time-consuming activity should be queued up using the 
 * <code>queuedExecutor</code>, which is similar to the SwingWorker concept.
 */
public class IOCDocument extends XalInternalDocument {

	/** The main data model of this application */
	private IOCModel _model;

    // Hold reference to parent desktop frame. Used to put up busy icon during work.
    JFrame _appFrame;

	/** Create a new empty document */
    public IOCDocument() {
        this(null);
    }

	/** Create a new document from url */
    public IOCDocument(final URL url) {
        //setSource(url);
        _model = new IOCModel();
        _appFrame = ((DesktopApplication)Application.getApp()).getDesktopFrame();
        // don't have anything here yet
    }

    public void saveDocumentAs(final URL url) {
        return;
    }

    /**
     * Specifies the document file suffixes supported by this document.
     * @return An array of file suffixes corresponding to writable files
     */
    public String[] writableDocumentTypes() {
        return new String[] {};
	}

    /**
	 * Overriding default window title to my liking.
     * @param newTitle The new title for this document.
     */
    public void setTitle( final String newTitle ) {
        String modifiedTitle = "idtioc" + newTitle;
		super.setTitle( modifiedTitle );
        _documentListenerProxy.titleChanged( this, title );
    }	

    /**
     * Queue up job for background thread to gather the current set of IOC
     * objects. When complete, fire off <code>IOCModelEvent</code> to notify GUI
     * that new list of iocs is available for display. 
     */
    public void loadInitialIOCData() {

        QueuedExecutorUtil.execute(new SwingThreadWork(_appFrame) {
                
                public void doNonUILogic() {
                    // Get current set of IOC objects, and put into IOCModel
                    IOCDAO iocDAO = null;
                    IOCAttributeTypeDAO iocAttrTypeDAO;
                    List iocList = null;
                    List iocAttrTypeList = null;
                    try {
                        // get list of ioc's
                        iocDAO = new IOCDAO();
                        iocList = iocDAO.findIOCs();

                        // get list of possible attribute types
                        iocAttrTypeDAO = new IOCAttributeTypeDAO();
                        iocAttrTypeList = iocAttrTypeDAO.findIOCAttributeTypes();
                        
                    } catch (DAOException de) {
                        de.printStackTrace();
                        // log error
                    }
                    _model.setIocList(iocList);
                    _model.setIocAttributeTypeList(iocAttrTypeList);
                }
                
                public void doUIUpdateLogic() {
                    // fire off notification of modification to listeners
                    _model.notifyIOCModelListeners(new IOCModelEvent(IOCModelEvent.NEW_IOC_LIST));
                }
            });
    }
    
    /**
     * Make a main window by instantiating the my custom window.
     */
    public void makeMainWindow() {
        _mainWindow = new IOCWindow(this);
        loadInitialIOCData();
    }

    /**
     * Programatically selects an ioc for display. Has the same effect as a user
     * clicking an ioc from the ioc list in the GUI. Provides means for external
     * document to command us to display a particular ioc.
     *
     * @param iocName ioc to select
     */
    public void actionSelectIoc(String iocName) {
        final String selectedIocName = iocName;
        QueuedExecutorUtil.execute(new SwingThreadWork(_appFrame) {
                
                public void doNonUILogic() {
                    // find IOC object by given ioc name
                    List iocList = _model.getIocList();
                    IOC selectedIoc = null;
                    if (iocList != null) {
                        Iterator iocIt = iocList.iterator();
                        while (iocIt.hasNext()) {
                            IOC ioc = (IOC)iocIt.next();
                            if (ioc.getIocName().equals(selectedIocName)) {
                                selectedIoc = ioc;
                                break;
                            }
                        }
                    }
                    _model.setSelectedIoc(selectedIoc);
                }
                
                public void doUIUpdateLogic() {
                    // fire off notification of modification to listeners
                    _model.notifyIOCModelListeners(new IOCModelEvent(IOCModelEvent.NEW_IOC_SELECTION));
                }
            });
    }
    
    /**
     * Queue up job for background thread to gather the current set of IOC
     * attributes. When complete, fire off <code>IOCModelEvent</code> to notify GUI
     * that new set of ioc attributes is available for display. 
     */
    public void actionAttributeSearch() {
        QueuedExecutorUtil.execute(new SwingThreadWork(_appFrame) {
                
                public void doNonUILogic() {
                    IOC selectedIoc = _model.getSelectedIoc();
                    
                    // force read of lazy attributes from db
                    if (selectedIoc != null)
                        selectedIoc.getAttributes().iterator();
                }
                
                public void doUIUpdateLogic() {
                    // fire off notification of modification to listeners
                    _model.notifyIOCModelListeners(new IOCModelEvent(IOCModelEvent.NEW_IOC_ATTR_LIST));
                }
            });
    }

    /**
     * Queue up job to update/save given ioc object. Notify GUI when complete.
     */
    public void actionSaveIoc(IOC ioc) {
        final IOC saveThisIoc = ioc;
        QueuedExecutorUtil.execute(new SwingThreadWork(_appFrame) {
                
                public void doNonUILogic() {
                    if (saveThisIoc != null) {
                        IOCDAO iocDAO = null;
                        try {
                            iocDAO = new IOCDAO();
                            iocDAO.save(saveThisIoc);
                        } catch (DAOException de) {
                            de.printStackTrace();
                            // log error
                        }
                        // add ioc to model if it is new
                        List iocList = _model.getIocList();
                        if (!iocList.contains(saveThisIoc))
                            iocList.add(saveThisIoc);

                    }
                }
                
                public void doUIUpdateLogic() {
                    // fire off notification of modification to listeners
                    _model.notifyIOCModelListeners(new IOCModelEvent(IOCModelEvent.IOC_SAVED));
                }
            });

    }

    /**
     * Queue up job to delete given ioc attribute object. Notify GUI when complete.
     */
    public void actionDeleteIocAttribute(IOCAttribute iocAttr) {
        final IOCAttribute deleteThisIocAttr = iocAttr;
        QueuedExecutorUtil.execute(new SwingThreadWork(_appFrame) {
                
                public void doNonUILogic() {
                    if (deleteThisIocAttr != null) {
                        IOCAttributeDAO iaDAO = null;
                        try {
                            iaDAO = new IOCAttributeDAO();

                            // delete attribute from ioc object
                            IOC ioc = deleteThisIocAttr.getIoc();
                            ioc.removeAttribute(deleteThisIocAttr);

                            // delete attribute itself
                            iaDAO.delete(deleteThisIocAttr);

                            _model.setSelectedIocAttribute(null);

                        } catch (DAOException de) {
                            de.printStackTrace(System.out);
                        }
                    }
                }
                public void doUIUpdateLogic() {
                    // fire off notification of modification to listeners
                    _model.notifyIOCModelListeners(new IOCModelEvent(IOCModelEvent.IOC_ATTRIBUTE_DELETED));
                }
            });

    }


	/**
	 * Get the launch model which represents the main model of this document
	 * @return the main model of this document
	 */
	public IOCModel getModel() {
		return _model;
	}

    private IOCWindow myWindow() {
        return (IOCWindow)_mainWindow;
    }
}
