/*
   Copyright (c) 2004-2005 The University of Chicago, as Operator
   of Argonne National Laboratory.
*/
package gov.anl.aps.irmis.apps.pv.cfw;

import java.net.URL;
import java.util.List;
import java.util.Set;
import java.util.Iterator;
import javax.swing.SwingUtilities;
import javax.swing.JDesktopPane;
import javax.swing.JFrame;

// XAL framework stuff
import gov.sns.application.*;

// needed for Session.clear()
import net.sf.hibernate.HibernateException;

// general app utilities
import gov.anl.aps.irmis.apps.SwingThreadWork;
import gov.anl.aps.irmis.apps.QueuedExecutorUtil;

// other applications
import gov.anl.aps.irmis.apps.ioc.cfw.IOCDocument;

// rdbCore imports
import gov.anl.aps.irmis.persistence.HibernateUtil;  // needed for Session.clear()
import gov.anl.aps.irmis.persistence.pv.IOCBootDAO;
import gov.anl.aps.irmis.persistence.pv.RecordDAO;
import gov.anl.aps.irmis.persistence.pv.FieldDAO;
import gov.anl.aps.irmis.persistence.pv.Record;
import gov.anl.aps.irmis.persistence.pv.RecordType;
import gov.anl.aps.irmis.persistence.pv.RecordTypeDAO;
import gov.anl.aps.irmis.persistence.pv.Field;
import gov.anl.aps.irmis.persistence.pv.FieldType;
import gov.anl.aps.irmis.persistence.pv.RecordClientDAO;
import gov.anl.aps.irmis.persistence.DAOException;


/**
 * The primary controller class for IRMIS PV Viewer application. 
 * Provides "action" methods to query the IRMIS database for pv-related data.
 * These action methods are typically invoked by the GUI in <code>PVWindow</code>.
 * Any time-consuming activity should be queued up using the 
 * <code>queuedExecutor</code>, which is similar to the SwingWorker concept.
 */
public class PVDocument extends XalInternalDocument {

	/** The main data model of this application */
	private PVModel _model;

    // Hold reference to parent desktop frame. Used to put up busy icon during work.
    JFrame _appFrame;

	/** Create a new empty document */
    public PVDocument() {
        this(null);
    }

    public PVDocument(final URL url) {
        //setSource(url);
        _model = new PVModel();
        _appFrame = ((DesktopApplication)Application.getApp()).getDesktopFrame();
        // don't have anything here yet
    }

    public void saveDocumentAs(final URL url) {

        return;
    }

    /**
     * Specifies the document file suffixes supported by this document.
     * @return An array of file suffixes corresponding to writable files
     */
    public String[] writableDocumentTypes() {
        return new String[] {};
	}

    /**
	 * Overriding default window title to my liking.
     * @param newTitle The new title for this document.
     */
    public void setTitle( final String newTitle ) {
        String modifiedTitle = "idtpv" + newTitle;
		super.setTitle( modifiedTitle );
        _documentListenerProxy.titleChanged( this, title );
    }	
    
    /**
     * Queue up job for background thread to gather the current set of IOCBoot
     * objects, and the unique list of RecordType. When complete, fire off 
     * <code>PVModelEvent</code> to notify GUI that new list of iocs, db 
     * files, and record types is available for display.
     */
    public void loadInitialPVData() {
        QueuedExecutorUtil.execute(new SwingThreadWork(_appFrame) {
                
                public void doNonUILogic() {
                    // Get current set of IOCBoot objects, and put into PVModel
                    // Also get a unique list of current record types.
                    IOCBootDAO ibDAO = null;
                    List bootList = null;
                    RecordTypeDAO rtDAO = null;
                    List rtList = null;
                    try {
                        ibDAO = new IOCBootDAO();
                        bootList = ibDAO.findCurrentLoads();
                        rtDAO = new RecordTypeDAO();
                        rtList = rtDAO.findAllRecordTypes();
                    } catch (DAOException de) {
                        de.printStackTrace();
                        // log error
                    }
                    _model.setIocBootList(bootList);
                    _model.setRecordTypeList(rtList);
                }
                
                public void doUIUpdateLogic() {
                    // fire off notification of modification to listeners
                    _model.notifyPvModelListeners(new PVModelEvent(PVModelEvent.NEW_IOC_BOOT_LIST));
                }
            });
    }

            
    /**
     * Make the window for this document.
     */
    public void makeMainWindow() {

        _mainWindow = new PVWindow(this);

        // Queue up on background thread to get initial data from db
        // GUI will have busy cursor during this intialization
        loadInitialPVData();
    }

	
	/**
	 * Get the main model of this document
	 * @return the main model of this document
	 */
	public PVModel getModel() {
		return _model;
	}

    /**
     * Conduct search for process variables using given search parameters. Work queued
     * up on background thread, after which <code>PVModel</code> listeners are notified.
     *
     * @param pvSearchParams container for a variety of search constraints
     */
    public void actionPVSearch(PVSearchParameters pvSearchParams) {
        _model.setPVSearchParameters(pvSearchParams);
        QueuedExecutorUtil.execute(new SwingThreadWork(_appFrame) {
                public void doNonUILogic() {
                    
                    // proceed with pv search
                    RecordDAO rDAO = null;
                    PVSearchParameters pvSearchParams = _model.getPVSearchParameters();
                    List recordList = null;
                    try {
                        // Clear the hibernate cache of previous results, since our searches
                        // can potentially bloat the cache with thousands of objects.
                        rDAO = new RecordDAO();
                        rDAO.evict(_model.getRecordList());
                        rDAO.evict(_model.getLinkList());

                        // set up search constraints
                        rDAO.setBootIdConstraints(pvSearchParams.getIocBootIds());
                        rDAO.setRecordTypeConstraints(pvSearchParams.getRecTypes());
                        rDAO.setFieldIocResourceIdConstraints(pvSearchParams.getIocResourceIds());
                        rDAO.setRecordNameGlobConstraint(pvSearchParams.getRecNameGlob());
                        rDAO.setFieldNameGlobConstraint(pvSearchParams.getFieldNameGlob());
                        rDAO.setFieldValueGlobConstraint(pvSearchParams.getFieldValueGlob());
                        // do search
                        recordList = rDAO.findByConstraints();
                    } catch (DAOException de) {
                        de.printStackTrace();
                        // log error
                    }
                    _model.setRecordList(recordList);
                    
                    // clear any previous data
                    _model.setLinkList(null);
                    _model.setSelectedRecord(null);
                }
                public void doUIUpdateLogic() {
                    // fire off notification of modification to listeners
                    _model.notifyPvModelListeners(new PVModelEvent(PVModelEvent.NEW_RECORD_LIST));
                }
                
            });
    }

    /**
     * Conduct search for a particular record using search parameters with just the
     * full record name set. Work queued up on background thread, after which 
     * <code>PVModel</code> listeners are notified.
     *
     * @param recordName name of pv to search for (with or without trailing field name and modifiers)
     */
    public void actionSinglePVSearch(String recordName) {
        final String selectedRecordName = recordName;
        QueuedExecutorUtil.execute(new SwingThreadWork(_appFrame) {
                public void doNonUILogic() {
                    // update model in separate thread
                    RecordDAO rDAO = null;
                    List recordList = null;
                    try {
                        rDAO = new RecordDAO();
                        rDAO.setRecordNameGlobConstraint(PVUtils.basePVName(selectedRecordName));
                        recordList = rDAO.findByConstraints();
                    } catch (DAOException de) {
                        de.printStackTrace();
                        // log error
                    }
                    if (recordList != null && recordList.size() > 0)
                        _model.setSelectedRecord((Record)recordList.get(0));
                    else
                        _model.setSelectedRecord(null);
                }
                public void doUIUpdateLogic() {
                    // fire off notification of modification to listeners
                    _model.notifyPvModelListeners(new PVModelEvent(PVModelEvent.NEW_RECORD));
                }
                
            });
    }

    /**
     * Conduct search for fields of a given record. We don't conduct an explict
     * search here, but rather let lazy instantiation get the fields. We just need
     * to invoke a single call to selectedRecord.getFields(). We do need to explicitly
     * find all field types for the given record type, though, and then add any
     * that are missing to the selectedRecords field set. This is the logic that
     * handles the case where a records fields are partially defined by the dbd, and
     * then overridden by the db file. Lastly, we request
     * notification of any model listeners that new field data is here.
     */
    public void actionFieldSearch() {
        QueuedExecutorUtil.execute(new SwingThreadWork(_appFrame) {
                public void doNonUILogic() {
                    // getFields will cause sql query to db
                    Record selectedRecord = _model.getSelectedRecord();
                    Set fields = selectedRecord.getFields();
                    // query db (implied) to get all field types defined for given record type
                    Set fieldTypes = selectedRecord.getRecordType().getFieldTypes();
                    
                    // Iterate over field types, adding new fields as needed to
                    // selectedRecord. Here is where we determine field state.
                    Iterator rtftIt = fieldTypes.iterator();
                    while (rtftIt.hasNext()) {
                        FieldType rtft = (FieldType)rtftIt.next();
                        boolean found = false;
                        Iterator fIt = fields.iterator();
                        while (fIt.hasNext()) {
                            Field f = (Field)fIt.next();
                            
                            if (f.getFieldType().getFieldType()
                                .equals(rtft.getFieldType())) {
                                found = true;
                                if (f.getFieldState() == Field.UNKNOWN) {
                                    if (rtft.getDefaultFieldValue().equals(f.getFieldValue())) {
                                        f.setFieldState(Field.OVERWRITTEN);
                                    } else {
                                        f.setFieldState(Field.USER_DEFINED);
                                    }
                                }
                                break;
                            }
                        }
                        if (!found) {
                            Field newField = new Field();
                            newField.setFieldType(rtft);
                            newField.setRecord(selectedRecord);
                            newField.setFieldState(Field.DEFAULT);
                            newField.setFieldValue(rtft.getDefaultFieldValue());
                            // add to record, although this will not be persisted
                            // beyond current session.
                            selectedRecord.addField(newField);
                        }
                    }
                    
                }
                public void doUIUpdateLogic() {
                    // fire off notification of modification to listeners
                    _model.notifyPvModelListeners(new PVModelEvent(PVModelEvent.NEW_FIELD_LIST));
                }
                
            });
    }

    /**
     * Conduct search for any fields (in other records) that link to the given record.
     * Then notify any model listeners that new link data is here.
     */
    public void actionLinkSearch() {
        QueuedExecutorUtil.execute(new SwingThreadWork(_appFrame) {
                public void doNonUILogic() {
                    Record selectedRecord = _model.getSelectedRecord();
                    List iocBootList = _model.getIocBootList();
                    FieldDAO fDAO = null;
                    List fieldList = null;
                    try {
                        fDAO = new FieldDAO();
                        fieldList = 
                            fDAO.findByRecordNameAndLink(iocBootList,
                                                         selectedRecord.getRecordName());
                    } catch (DAOException de) {
                        de.printStackTrace();
                        // log error
                    }                        
                    _model.setLinkList(fieldList);
                }
                public void doUIUpdateLogic() {
                    // fire off notification of modification to listeners
                    _model.notifyPvModelListeners(new PVModelEvent(PVModelEvent.NEW_LINK_LIST));
                }
                
            });
    }

    /**
     * Conduct search for any clients of a given pv name.
     *
     */
    public void actionClientSearch() {
        QueuedExecutorUtil.execute(new SwingThreadWork(_appFrame) {
                public void doNonUILogic() {
                    Record selectedRecord = _model.getSelectedRecord();                        
                    RecordClientDAO rcDAO = null;
                    List recordClientList = null;
                    try {
                        rcDAO = new RecordClientDAO();
                        recordClientList = rcDAO.findByRecordName(selectedRecord.getRecordName());
                    } catch (DAOException de) {
                        de.printStackTrace();
                        // log error
                    }
                    _model.setClientList(recordClientList);
                }
                public void doUIUpdateLogic() {
                    // fire off notification of modification to listeners
                    _model.notifyPvModelListeners(new PVModelEvent(PVModelEvent.NEW_CLIENT_LIST));
                }
                
            });
    }

    /**
     * Pop up the IOC Viewer application on the desktop, preselected for
     * the ioc of our currently selected record.
     */
    public void actionProduceIOCDocument() {
        Application app = Application.getApp();
        List docs = app.getDocuments();
        
        // first see if it is already on desktop 
        Iterator docIt = docs.iterator();
        IOCDocument iocDoc = null;
        while (docIt.hasNext()) {
            XalInternalDocument doc = (XalInternalDocument)docIt.next();
            if (doc.getTitle().equals("IOC Viewer")) {
                iocDoc = (IOCDocument)doc;
                break;
            }
        }
        if (iocDoc == null) {
            iocDoc = new IOCDocument();
            app.produceDocument(iocDoc);
        }

        // now tell the ioc doc to preselect for our ioc
        String iocName = _model.getSelectedRecord().getIocBoot().getIoc().getIocName();
        iocDoc.actionSelectIoc(iocName);
    }


    /**
	 * Convenience method for getting the main window cast to the proper subclass of XalWindow.
     * This allows me to avoid casting the window every time I reference it.
     * @return The main window cast to its dynamic runtime class
     */
    private PVWindow myWindow() {
        return (PVWindow)_mainWindow;
    }
}
