/*
   Copyright (c) 2004-2005 The University of Chicago, as Operator
   of Argonne National Laboratory.
*/
package gov.anl.aps.irmis.apps.pv.cfw;

import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.HashMap;
import java.util.Map;

import gov.anl.aps.irmis.persistence.pv.Record;

/**
 * Application data model for IRMIS PV Viewer. Stores graph of current set of ioc's
 * and related data. Stores record search result set, and the individual record
 * of interest. A list of listeners interested in this data is maintained, and
 * notified as requested.
 */
public class PVModel {

    // list of others interested in changes to the data model
    private List pvModelListenersList = new ArrayList();

    // pv subgraph rooted in list of current IOCBoot objects
    private List iocBootList; 

    // list of unique RecordType for populating record type search list box
    private List recordTypeList;

    // pv subgraph root in list of searched Record objects
    private List recordList;

    /** 
        Stores same data as recordList, but used when recordList may have
        repeated entries with the same record name. For example, when retrieving
        the history of a pv or set of pvs. Here there will be one hash key
        for each record name, with each entry value being a list of ocurrences
        of the given pv.
    */
    private HashMap uniqueRecordMap;

    // reference to current record-of-interest
    private Record selectedRecord;

    // the list of pv ocurrences from uniqueRecordMap corresponding to selectedRecord
    private List selectedRecordHistory;

    // list of fields that have reference (link) to record-of-interest
    private List linkList;

    // list of pv clients taht have reference to record-of-interest
    private List clientList;
    
    // pv search parameters from view
    private PVSearchParameters searchParams;

	/**
	 * Do nothing constructor
	 */
	public PVModel() {
	}

    public void addPvModelListener(PVModelListener l) {
        pvModelListenersList.add(l);
    }

    public void notifyPvModelListeners() {
        notifyPvModelListeners(null);
    }

    /**
     * Notify registered listeners of the given <code>PVModelEvent</code>.
     *
     * @param modelEvent encapsulates one of several possible events
     */
    public void notifyPvModelListeners(PVModelEvent modelEvent) {
        Iterator it = pvModelListenersList.iterator();
        while (it.hasNext()) {
            PVModelListener l = (PVModelListener)it.next();
            l.modified(modelEvent);
        }
    }

    public List getIocBootList() {
        return iocBootList;
    }
    public void setIocBootList(List iocBootList) {
        this.iocBootList = iocBootList;
    }

    public List getRecordTypeList() {
        return recordTypeList;
    }
    public void setRecordTypeList(List recordTypeList) {
        this.recordTypeList = recordTypeList;
    }

    public List getRecordList() {
        return recordList;
    }
    public void setRecordList(List recordList) {
        this.recordList = recordList;
        // build up uniqueRecordMap as well
        if (recordList != null) {
            uniqueRecordMap = new HashMap();
            Iterator recordIt = recordList.iterator();
            while (recordIt.hasNext()) {
                Record rec = (Record)recordIt.next();
                
                // add record to ArrayList of existing map entry
                if (uniqueRecordMap.containsKey(rec.getRecordName())) {
                    ArrayList chain = (ArrayList)uniqueRecordMap.get(rec.getRecordName());
                    chain.add(rec);
                    
                } else {  // create new ArrayList and map entry
                    ArrayList newChain = new ArrayList();
                    newChain.add(rec);
                    uniqueRecordMap.put(rec.getRecordName(),newChain);
                }
            }
        } else {
            uniqueRecordMap = null;
        }
    }

    public Map getUniqueRecordMap() {
        return this.uniqueRecordMap;
    }

    public Record getSelectedRecord() {
        return selectedRecord;
    }
    public void setSelectedRecord(Record rec) {
        selectedRecord = rec;
    }

    public List getSelectedRecordHistory() {
        return selectedRecordHistory;
    }

    public void setSelectedRecordHistory(List history) {
        selectedRecordHistory = history;
    }

    public List getLinkList() {
        return linkList;
    }
    public void setLinkList(List linkList) {
        this.linkList = linkList;
    }

    public List getClientList() {
        return clientList;
    }
    public void setClientList(List clientList) {
        this.clientList = clientList;
    }

    public PVSearchParameters getPVSearchParameters() {
        return this.searchParams;
    }
    public void setPVSearchParameters(PVSearchParameters searchParams) {
        this.searchParams = searchParams;
    }

}
