/*
   Copyright (c) 2004-2005 The University of Chicago, as Operator
   of Argonne National Laboratory.
*/
package gov.anl.aps.irmis.persistence;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import net.sf.hibernate.Hibernate;
import net.sf.hibernate.HibernateException;
import net.sf.hibernate.Session;
import net.sf.hibernate.Criteria;
import net.sf.hibernate.type.Type;

/**
 * Base class for all DAOs (Data Access Objects). Provides some commonly required
 * methods to query and store objects, as well as session methods such as
 * rollback and commit.
*/
public abstract class BaseDAO {


    /**
     * Constructor initiates or joins current thread session transaction.
     */
    public BaseDAO() throws DAOException {
        try {
            HibernateUtil.beginTransaction();
        } catch (HibernateException he) {
            throw new DAOException(he);
        }
    }

    /**
     * Removes the object from the database with specified class
     * type and <code>id</code>.
     *
     * @param c the class type to remove
     * @param id the id of the class type
     * @throws DAOException
     */
    protected void removeObj(Class c, Long id) throws DAOException {
        try {
            Session session = HibernateUtil.getSession();
            // first load the object with the current session.
            // the object must be loaded in this session before it
            //   is deleted.
            Object obj = session.load(c, id);
            session.delete(obj);
            session.flush();
            HibernateUtil.commitTransaction();
        }
        catch(Exception e) {
            e.printStackTrace(System.out);
            try {
                HibernateUtil.rollbackTransaction();
            } catch (HibernateException he) {
                he.printStackTrace(System.out);
            }
            throw new DAOException(e);
        }
    }

    /**
     * Removes the object from the database. Assumes the object has already
     * been introduced into the session through a prior retrieve.
     *
     * @param o the data object to remove
     * @throws DAOException
     */
    protected void removeObj(Object obj) throws DAOException {
        try {
            Session session = HibernateUtil.getSession();
            session.delete(obj);
            session.flush();
            HibernateUtil.commitTransaction();
        }
        catch(Exception e) {
            e.printStackTrace(System.out);
            try {
                HibernateUtil.rollbackTransaction();
            } catch (HibernateException he) {
                he.printStackTrace(System.out);
            }
            throw new DAOException(e);
        }
    }

    /**
     * Creates a hibernate Criteria object for a given mapped hibernate class c.
     *
     * @param c the mapped hibernate class
     * @return hibernate Criteria object
     */
    protected Criteria createCriteria(Class c) throws DAOException {
        Criteria criteria = null;
        try {
            Session session = HibernateUtil.getSession();
            criteria = session.createCriteria(c);
        }
        catch(HibernateException he) {
            he.printStackTrace(System.out);
            throw new DAOException(he);
        }
        return criteria;
    }

    /**
     * Retrieves and <code>Object</code> of the class type specified
     * by <code>c</code>, and having the given <code>id</code>.
     *
     * @param c the class to load
     * @param id the id of the class
     * @return Object may be null if object with ID doesn't exist
     * @throws DAOException
     */
    protected Object retrieveObj(Class c, Long id) throws DAOException {
        Object obj = null;

        try {
            Session session = HibernateUtil.getSession();
            obj = session.load(c, id);
        }
        catch(HibernateException he) {
            he.printStackTrace(System.out);
            throw new DAOException(he);
        }
		return obj;
    }

    /**
     * Retrieves an <code>Object</code> from the database.
     *
     * @param query the HQL query
     * @param values the value(s) that is inserted into the query.  May be null
     * if the desired query does not take a parameter.
     * @param types the hibernate types of the above values
     * @return Object object retrieved via query
     * @throws DAOException
     */
    protected Object retrieveObj(String query, Object[] values, Type[] types) throws DAOException {
        List objects = retrieveObjs(query, values, types);
        if (objects != null) {
            if (objects.size() == 0) {
                return null;
            }
            else {
                return objects.get(0);
            }
        }
        else {
            return null;
        }
    }

    /**
     * Retrieves a <code>List</code> of <code>Object</code>s from the database.
     *
     * @param query the HQL query
     * @param values the value(s) that is inserted into the query.  May be null
     * if the desired query does not take a parameter.
     * @param types the hibernate types of the above values
     * @return List will be null if no objects are retrieved
     * @throws DAOException
     */
    protected List retrieveObjs(String query, Object[] values, Type[] types) throws DAOException {
        List results = null;

        try {
            Session session = HibernateUtil.getSession();
            if (values != null) {
                results = (List) session.find(query, values, types);
            }
            else {
                results = (List) session.find(query);
            }
        }
        catch (HibernateException he) {
            he.printStackTrace(System.out);
            throw new DAOException(he);
        }
		return results;
    }

    /**
     * Stores <code>obj</code>, making it persistent.
     *
     * @param obj
     * @throws DAOException
     */
    protected void storeObj(Object obj) throws DAOException {
        try {
            Session session = HibernateUtil.getSession();
            session.saveOrUpdate(obj);
            session.flush();
            HibernateUtil.commitTransaction();
        }
        catch(HibernateException he) {
            try {
                HibernateUtil.rollbackTransaction();
            } catch (HibernateException he2) {
                he2.printStackTrace(System.out);
            }
            throw new DAOException(he);
        }
    }

    /**
     * Retrieves current session, or creates new one if it doesn't exist yet.
     *
     * @return hibernate Session object
     * @throws DAOException
     */
    public Session getSession() throws DAOException {
        Session s = null;
        try {
            s = HibernateUtil.getSession();
        } catch (HibernateException he) {
            he.printStackTrace(System.out);
            throw new DAOException(he);
        }
        return s;
    }

    /**
     * Closes the current session. Not typically invoked until application
     * or servlet request is ending.
     */
    protected void closeSession() {
        HibernateUtil.closeSession();
    }

    /**
     * Evicts given object from Hibernate first-level cache, and any associated
     * objects (if they are mapped with cascade="all". Ideally we shouldn't have
     * a hibernate-specific call in the DAO API, but this can be considered a
     * generic cache evict for other underlying DAO providers.
     *
     * @param o data object that was returned by a hibernate query
     * @throws DAOException
     */
    public void evict(Object o) throws DAOException {
        if (o != null) {
            try {
                Session session = HibernateUtil.getSession();
                session.evict(o);
            } catch (HibernateException he) {
                he.printStackTrace(System.out);
                throw new DAOException(he);
            }
        }
    }

    /**
     * Evicts given collection from Hibernate first-level cache, and any associated
     * objects (if they are mapped with cascade="all". Ideally we shouldn't have
     * a hibernate-specific call in the DAO API, but this can be considered a
     * generic cache evict for other underlying DAO providers.
     *
     * @param c collection of data objects that was returned by a hibernate query
     * @throws DAOException
     */
    public void evict(Collection c) throws DAOException {
        if (c != null) {
            try {
                Session session = HibernateUtil.getSession();
                Iterator it = c.iterator();
                while (it.hasNext()) {
                    Object o = it.next();
                    session.evict(o);
                }
            } catch (HibernateException he) {
                he.printStackTrace(System.out);
                throw new DAOException(he);
            }


        }
    }

    /**
     * Rollback the current session's database transaction. Closes the
     * session afterwards.
     *
     * @throws DAOException
     */
    protected void rollbackTransaction() throws DAOException {
        try {
            HibernateUtil.rollbackTransaction();
        } catch (HibernateException he) {
            throw new DAOException(he);
        }
    }

    /**
     * Commit the current session's database transaction.
     *
     * @throws DAOException
     */
    protected void commitTransaction() throws DAOException {
        try {
            HibernateUtil.commitTransaction();
        } catch (HibernateException he) {
            throw new DAOException(he);
        }
    }

}
