/*
   Copyright (c) 2004-2005 The University of Chicago, as Operator
   of Argonne National Laboratory.
*/
package gov.anl.aps.irmis.persistence.pv;

import gov.anl.aps.irmis.persistence.HashCodeUtil;
import java.io.Serializable;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;

/**
 * IRMIS business object that represents a single EPICS IOC.
 * This class is mapped via hibernate almost one-to-one with IRMIS ioc table.
 */
public class IOC implements Serializable {

    private Long id;
    private Set iocAttributes = new HashSet();
    private String iocName;
    private boolean active;
    private String system;
    private Date modifiedDate;
    private String modifiedBy;

    /**
     * Constructor
     */
    public IOC() {
    }

    /**
     * Get primary key.
     */
    public Long getId() {
        return this.id;
    }

    /**
     * Primary key. Not typically set explicitly by application code.
     */
    public void setId(Long id) {
        this.id = id;
    }

    /**
     * Get the complete set of IOCAttributes associated with this ioc.
     * This set represents the set of descriptive attributes of this ioc.
     *
     * @see IOCAttribute
     */
    public Set getAttributes() {
        return this.iocAttributes;
    }
    /**
     * Set the complete set of IOCAttributes associated with this ioc.
     *
     * @see IOCAttribute
     */
    public void setAttributes(Set value) {
        this.iocAttributes = value;
    }
    /**
     * Add one IOCAttribute to set of known attributes. Establishes bidirectional
     * identity (ie. attribute knows which IOC it belongs to as well).
     */
    public void addAttribute(IOCAttribute iocAttribute) {
        iocAttribute.setIoc(this);
        iocAttributes.add(iocAttribute);
    }
    /**
     * Remove one IOCAttribute from set of known attributes.
     */
    public void removeAttribute(IOCAttribute iocAttribute) {
        iocAttributes.remove(iocAttribute);
    }

    /**
     * Get the ioc's name. Example: iocbbpm1 or iocacis.
     */
    public String getIocName() {
        return this.iocName;
    }
    /**
     * Set the ioc's name. Example: iocbbpm1 or iocacis.
     */    
    public void setIocName(String iocName) {
        this.iocName = iocName;
    }

    /**
     * Indicates whether this ioc is considered in use or not. There may still
     * be other database entities which refer to an ioc which has been taken out
     * of commission from a controls standpoint. 
     */
    public boolean getActive() {
        return this.active;
    }
    public void setActive(boolean active) {
        this.active = active;
    }

    /**
     * An arbitrary system name under which this ioc belongs.
     */
    public String getSystem() {
        return this.system;
    }
    public void setSystem(String system) {
        this.system = system;
    }

    /**
     * Direct reflection of last modification of underlying database row.
     */
    public Date getModifiedDate() {
        return this.modifiedDate;
    }
    public void setModifiedDate(Date value) {
        this.modifiedDate = value;
    }

    /**
     * Represent user name or application name which last modified (created)
     * underlying database row.
     */
    public String getModifiedBy() {
        return this.modifiedBy;
    }
    public void setModifiedBy(String value) {
        this.modifiedBy = value;
    }

    public String toString() {
        return getId().toString() + ":" + getIocName();
    }

    /**
     * Custom equals which compares "business key" equality.
     */
    public boolean equals(Object other) {
        if (this==other) return true;
        if ( !(other instanceof IOC) ) return false;
        final IOC castOther = (IOC) other;
        return this.getIocName().equals(castOther.getIocName());
    }

    /**
     * Custom hashCode computed on "business key" fields.
     */
    public int hashCode() {
        int result = HashCodeUtil.SEED;
        result = HashCodeUtil.hash(result, getIocName());
        return result;
    }

}
